#include "StdAfx.h"
#include "trackball.h"
#include "glextmap.h"

CTrackball::CTrackball()
{
//	Yaw   = 0.0f;//0.5f;
//	Pitch = 0.0f;//-0.5;
//	Roll  = 0.0f;//0.0f;
	Yaw   = 0.5f;
	Pitch = -0.5;
	Roll  = 0.0f;
	Radius = 10.0f;

	loadP4(CamTarget, 0,0,0);

	q4_from_euler(&Quat, Pitch, Yaw, Roll);
	q4_to_m4(&XForm, &Quat);

	PanSpeed  = 0.003;
	RotSpeed  = 0.5f/90.0f;
	ZoomSpeed = 1.0f;

	MouseState = 10;
	parent = NULL;

	MouseState = 0;
	MouseClickX = 0;
	MouseClickY = 0;

}

void CTrackball::SetParent(HWND p)
{
	parent = p;
}

CTrackball::~CTrackball(void)
{
}

void CTrackball::OnMouseDown(int flags, int x, int y)
{
	MouseState = flags;
	MouseClickX = x;
	MouseClickY = y;
	if (parent!=NULL) 
		SetCapture(parent);
}

int CTrackball::OnMouseMove(int flags, int x, int y)
{
	int dx, dy;
	int alt, ctrl, shift, left, middle, right;

	ctrl = flags & MK_CONTROL;
	shift = flags & MK_SHIFT;
	left = flags & MK_LBUTTON;
	middle = flags & MK_MBUTTON;
	right = flags & MK_RBUTTON;
	alt = GetAsyncKeyState(VK_MENU);

	dx = x - MouseClickX;
	dy = y - MouseClickY;

	if ( /*alt &&*/ left)
	{
		Rotate(dx, dy);
		MouseClickX = x;
		MouseClickY = y;

		return 1;
	}

	if (alt && right)
	{
		Zoom(dx, dy);
		MouseClickX = x;
		MouseClickY = y;
		
		return 1;
	}

	if (alt && middle)
	{
		PanKey(dx, dy);
		MouseClickX = x;
		MouseClickY = y;

		return 1;
	}

	return 0;
}

void CTrackball::OnMouseUp(int flags, int x, int y)
{
	MouseState = 10;
	if (parent!=NULL) ReleaseCapture();
}

void CTrackball::ApplyXForm(void)
{
	q4_from_euler(&Quat, Pitch, Yaw, Roll);
	q4_to_m4(&XForm, &Quat);
	//MATRIX4x4 inv;
	//m4_inv(&inv, &XForm);

	glTranslatef(0, 0, -Radius);
	glMultMatrixf(XForm.raw);
	glTranslatef(CamTarget.v.x, CamTarget.v.y, CamTarget.v.z);

	MATRIX4x4 temp;
	glGetFloatv(GL_MODELVIEW_MATRIX, temp.raw);
	m4_inv(&ViewMatrix, &temp);
}

void CTrackball::GetCamPos(VECTOR4 *dest)
{
	//m4_axisPos(dest, &XForm);
	*dest = CamTarget;
	v4_mul_s(dest, dest, -1.0f);

	VECTOR4 z;
	m4_axisZ(&z, &XForm);
	v4_mul_s(&z, &z, Radius);
	v4_add_v4(dest, dest, &z);
}

MATRIX4x4& CTrackball::GetViewMatrix()
{
	return ViewMatrix;
}

void CTrackball::PanKey(int dx, int dy)
{
	// \todo: Pan kamere
	VECTOR4 hor, vert;

	m4_axisX(&hor, &XForm);
	m4_axisY(&vert, &XForm);
	float speedx = dx * PanSpeed;
	float speedy = dy * PanSpeed;

	CamTarget.v.x += hor.v.x * speedx;
	CamTarget.v.y += hor.v.y * speedx;
	CamTarget.v.z += hor.v.z * speedx;

	CamTarget.v.x -= vert.v.x * speedy;
	CamTarget.v.y -= vert.v.y * speedy;
	CamTarget.v.z -= vert.v.z * speedy;
}

void CTrackball::MoveForwardBackward(float s)
{
	VECTOR4 dir;

	m4_axisZ(&dir, &XForm);

	CamTarget.v.x += dir.v.x * s;
	CamTarget.v.y += dir.v.y * s;
	CamTarget.v.z += dir.v.z * s;
}

void CTrackball::Pan(int dx, int dy)
{
	// \todo: Pan kamere
	VECTOR4 hor, vert;

	m4_axisX(&hor, &XForm);
	m4_axisY(&vert, &XForm);
	float sRadius = abs(Radius)/100.0f;
	float speedx = dx * PanSpeed * sRadius;
	float speedy = dy * PanSpeed * sRadius;

	CamTarget.v.x += hor.v.x * speedx;
	CamTarget.v.y += hor.v.y * speedx;
	CamTarget.v.z += hor.v.z * speedx;

	CamTarget.v.x -= vert.v.x * speedy;
	CamTarget.v.y -= vert.v.y * speedy;
	CamTarget.v.z -= vert.v.z * speedy;
}

void CTrackball::Rotate(int dx, int dy)
{
		// \todo: Rotacija kamere
	Yaw   -= dx * RotSpeed;
	Pitch -= dy * RotSpeed;

	q4_from_euler(&Quat, Pitch, Yaw, Roll);
	q4_to_m4(&XForm, &Quat);
}

void CTrackball::Zoom(int dx, int dy)
{
		// \todo: Zoom kamere
	float val;
	float adx, ady;

	adx = abs(dx);
	ady = abs(dy);

	if (adx > ady) val = -dx;
	else val = dy;

	Radius += val * ZoomSpeed;
}

void CTrackball::Update()
{
	q4_from_euler(&Quat, Pitch, Yaw, Roll);
	q4_to_m4(&XForm, &Quat);
}