#include "Shader.h"

Shader::Shader(std::string &filename):m_fileName(filename)
{
  m_status = true;
  m_programSource = NULL;  
  m_shader = -1;
      
  LoadProgramSize(filename);
  if(m_size<=0)
  {
    m_status = false;
    return;
  }
  char *vert = strstr(filename.c_str(),".vert");
  if(vert)
    m_programType = PT_VERTEX_PROGRAM;
  else
  {
    char *frag = strstr(filename.c_str(),".frag");
    if(frag)
      m_programType = PT_FRAGMENT_PROGRAM;
    else
    {
      m_status = false;
      return;
    }
  }
 
  if(m_programType == PT_VERTEX_PROGRAM) 
    m_shader = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
  else
    m_shader = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);

  FILE *source = fopen(m_fileName.c_str(),"rb");
  m_programSource = new char[m_size];
  fread(m_programSource,m_size,1,source);
  fclose(source);
 
  glShaderSourceARB(m_shader, 1,(const char **)&m_programSource, &m_size);  
  glCompileShaderARB(m_shader);
  
  int compiled = 0, longitud = 0,laux = 0;
  glGetObjectParameterivARB(m_shader, GL_OBJECT_COMPILE_STATUS_ARB, &compiled); 
  glGetObjectParameterivARB(m_shader, GL_OBJECT_INFO_LOG_LENGTH_ARB , &longitud);
  if(longitud==1)
  {
    m_logString="";
    return;
  }
  
  GLcharARB *logString = new char [longitud];
  glGetInfoLogARB(m_shader, longitud, &laux, logString);
  m_logString.clear();
  for(int i=0;i<laux;i++)
    m_logString.push_back(logString[i]);
  delete []logString;

  if(!compiled)
  {
    m_status = false;
    return;          
  } 
}
//--------------------------------------------------------------------------------
Shader::Shader(std::string &name,std::string &source,EShaderType type)
{
  m_status = true;
  m_programSource = NULL;
  m_shader = -1;
  m_fileName = name;      
  m_size = (GLint) source.size();
  if(m_size<=0)
  {
    m_status = false;
    return;
  }
  m_programType = type;
  
  if(m_programType == PT_VERTEX_PROGRAM) 
    m_shader = glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
  else
    m_shader = glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);

  m_programSource = new char[m_size+1];
  memcpy(m_programSource,source.c_str(),m_size);
 
  glShaderSourceARB(m_shader, 1,(const char **)&m_programSource, &m_size);  
  glCompileShaderARB(m_shader);
  
  
  int compiled = 0, longitud = 0,laux = 0;
  glGetObjectParameterivARB(m_shader, GL_OBJECT_COMPILE_STATUS_ARB, &compiled); 
  glGetObjectParameterivARB(m_shader, GL_OBJECT_INFO_LOG_LENGTH_ARB , &longitud);
  if(longitud==1)
  {
    m_logString="";
    return;
  }
  GLcharARB *logString = new char [longitud];
  glGetInfoLogARB(m_shader, longitud, &laux, logString);
  m_logString.clear();
  for(int i=0;i<laux;i++)
    m_logString.push_back(logString[i]);

  delete [] logString;
  if(!compiled)
  {
    m_status = false;
    return;          
  } 
}
//--------------------------------------------------------------------------------
Shader::~Shader()
{
  if(m_shader!=-1)
    glDeleteObjectARB(m_shader);  
  if(m_programSource) delete m_programSource;
}
//--------------------------------------------------------------------------------
void Shader::LoadProgramSize(std::string &filename)
{
	int shader;
	m_size = -1;

	shader = _open(filename.c_str(),_O_RDONLY);
	if (shader == ENOENT)
		m_size = -1;
  
  m_size = _lseek(shader, 0, SEEK_END);
	_close(shader);
}
//--------------------------------------------------------------------------------
