#include "testcopypixels.h"
#include "shader.h"


#include "log.h"
#include <vector>

CLog Log;

#define GLCALL(a)		{(a); glerr(#a, __LINE__);}
//#define GLCALL(a)		a
static void glerr(char *str, int line)
{
	int err;

	err=glGetError();
	if (err!=0) 
	{
		Log.AddLine(__FILE__, "GLERR: [%5d]  %s: %s", line, str, gluErrorString(err));
		//MessageBox(NULL, gluErrorString(err), "EJJOJ!", MB_OK);
	}
}



#define CHECK_FRAMEBUFFER_STATUS()                               \
{                                                                \
	GLenum status;                                               \
	status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);    \
	switch(status) {                                             \
		  case GL_FRAMEBUFFER_COMPLETE_EXT:                      \
		  break;                                                 \
		  case GL_FRAMEBUFFER_UNSUPPORTED_EXT:                   \
		  Log.AddLine(__FILE__, "GL_FRAMEBUFFER_UNSUPPORTED_EXT"); \
		  break;                                                 \
		  default:                                               \
		  Log.AddLine(__FILE__, "ERROR %04x", status);           \
		  return false;                                          \
}}


GLuint fb, depth_rb, stencil_rb;
GLuint color_tex, depth_tex;


#define TEXX 512
#define TEXY 512

BYTE texture_data[TEXX * TEXY * 4];

HINSTANCE hInstance = NULL;
HWND hWnd = NULL;
HDC hDC = NULL;
HGLRC hRC = NULL;

char className[] = "FBO Test";
int WIDTH = 512;
int HEIGHT = 512;

float orbitRadius = 15.0f;
float pitch = 0.0f;
float yaw = 0.0f;
float roll = 0.0f;

float lyaw = 0.0f;


float tx = 0.0f;
float ty = 0.0f;

typedef struct tagVec3
{
	float x,y,z;
} tVec3;

tVec3 pivot={0,0,0};

GLfloat mv[16];

GLUquadric* gluSphereObj;

void OnResize (int width, int height);
void OnMouseMove (int x, int y);


GLSLShaderObject* shader;
CGLSLProgramObject* program;


//--------------------------------------------------------------------------
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine,int nCmdShow)
{
	MSG msg;
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			    = CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc  	= (WNDPROC)WndProc;
	wcex.cbClsExtra 		= 0;
	wcex.cbWndExtra  		= 0;
	wcex.hInstance	  	= hInstance;
	wcex.hIcon		    	= LoadIcon(hInstance, NULL);
	wcex.hCursor	  	  = LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName 	= className;
	wcex.lpszClassName	= className;
	wcex.hIconSm		    = LoadIcon(hInstance,NULL);
  
	RegisterClassEx(&wcex);
  hInstance = hPrevInstance;
  
  RECT rect = { 0, 0, WIDTH, HEIGHT};   
  AdjustWindowRectEx(&rect,WS_CAPTION | WS_SYSMENU | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,FALSE ,WS_EX_APPWINDOW);	  
  WIDTH = rect.right-rect.left;
  HEIGHT = rect.bottom-rect.top;
  hWnd = CreateWindowEx(WS_EX_APPWINDOW,className, className,
					    WS_OVERLAPPEDWINDOW,	
                      //WS_CAPTION | WS_SYSMENU | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,  
	                    50, 50, WIDTH,HEIGHT, NULL, NULL, hInstance, NULL);  
  
  if (!hWnd) return FALSE;
  ShowWindow(hWnd, SW_SHOW);
  UpdateWindow(hWnd);

  if(!InitOpenGLStuff()) 
  {
    MessageBox(NULL,"Error initializing OpenGL context & extensions","Error",MB_OK);
    return false;
  }

  bool done = FALSE;
  while(!done)
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{
			if (msg.message==WM_QUIT)
				done=TRUE;
			else
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
		else Render();
	}

	wglDeleteContext(hRC);
	return (int) msg.wParam;
}

bool MouseL=false, MouseM=false, MouseR=false;
int MouseClickX=0, MouseClickY=0;

//--------------------------------------------------------------------------
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{    
	case WM_LBUTTONUP:
		{
			MouseL = false;
			break;
		}
	case WM_RBUTTONUP:
		{
			MouseR = false;
			break;
		}
	case WM_MBUTTONUP:
		{
			MouseM = false;
			break;
		}

	case WM_LBUTTONDOWN:
		{
			MouseL = true;
			MouseClickX = LOWORD(lParam);
			MouseClickY = HIWORD(lParam);
			break;
		}
	case WM_RBUTTONDOWN:
		{
			MouseR = true;
			MouseClickX = LOWORD(lParam);
			MouseClickY = HIWORD(lParam);
			break;
		}
	case WM_MBUTTONDOWN:
		{
			MouseM = true;
			MouseClickX = LOWORD(lParam);
			MouseClickY = HIWORD(lParam);
			break;
		}
	case WM_MOUSEMOVE:
		{
			OnMouseMove(LOWORD(lParam), HIWORD(lParam));
			break;
		}

	case WM_CLOSE:
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	case WM_SIZE:
		{
			RECT r;
			GetClientRect(hWnd, &r);
			OnResize(r.right - r.left, r.bottom - r.top);		
			return 0;											
		}
		break;													
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// Purpose: transposes the given matrix mSrc and stores result into mDst (swap along (0,0)-(3,3), so (1,0) <-> (0,1)
void MatrixTranspose(GLfloat mSrc[16], GLfloat mDst[16])
{
	mDst[0] = mSrc[0]; mDst[1] = mSrc[4]; mDst[2] = mSrc[8]; mDst[3] = mSrc[12];
	mDst[4] = mSrc[1]; mDst[5] = mSrc[5]; mDst[6] = mSrc[9]; mDst[7] = mSrc[13];
	mDst[8] = mSrc[2]; mDst[9] = mSrc[6]; mDst[10] = mSrc[10]; mDst[11] = mSrc[14];
	mDst[12] = mSrc[3]; mDst[13] = mSrc[7]; mDst[14] = mSrc[11]; mDst[15] = mSrc[15];
}



void OnMouseMove(int x, int y)
{
	if (MouseL)
	{
		int dx = x - MouseClickX;
		int dy = y - MouseClickY;
		pitch += -dy/10.0f;
		yaw	  += -dx/10.0f;
		MouseClickX = x;
		MouseClickY = y;
	}
	if (MouseR)
	{
		int dx = x - MouseClickX;
		int dy = y - MouseClickY;
		int t;

		if (abs(dx) > abs(dy))	t = dx;
		else t = dy;

		orbitRadius += t/10.0f;

		MouseClickX = x;
		MouseClickY = y;
	}
	if (MouseM)
	{
		int dx = x - MouseClickX;
		int dy = y - MouseClickY;

		tx = -dx/10.0f;
		ty = dy/10.0f;

		GLfloat tmv[16];
		MatrixTranspose(mv, tmv);
		tVec3 vx ={tmv[0], tmv[1], tmv[2]};
		tVec3 vy ={tmv[4], tmv[5], tmv[6]};
		tVec3 vz ={tmv[8], tmv[9], tmv[10]};
		//camera->position = camera->position.getValue() + tx * vx + ty * vy;

		//omx = tx*vx
		tVec3 omx = {tx * vx.x, tx * vx.y, tx * vx.z};

		//omy = ty*vy
		tVec3 omy = {ty * vy.x, ty * vy.y, ty * vy.z};

		pivot.x = pivot.x + omx.x + omy.x;
		pivot.y = pivot.y + omx.y + omy.y;
		pivot.z = pivot.z + omx.z + omy.z;

		MouseClickX = x;
		MouseClickY = y;
	}
}

void OnResize (int width, int height)									
{
	glViewport (0, 0, (GLsizei)(width), (GLsizei)(height));				
	glMatrixMode (GL_PROJECTION);										
	glLoadIdentity ();													
	gluPerspective (45.0f, (GLfloat)(width)/(GLfloat)(height), 0.01f, 80.0f);		
	glMatrixMode (GL_MODELVIEW);										
	glLoadIdentity ();													
	WIDTH = width;
	HEIGHT = height;
}

//--------------------------------------------------------------------------
bool InitOpenGLStuff()
{
	Log.CreateFile("errors.log");

  // Init main window's rendering context
  hDC = GetDC(hWnd);
  PIXELFORMATDESCRIPTOR pfd =											// pfd Tells Windows How We Want Things To Be
  {
	  sizeof (PIXELFORMATDESCRIPTOR),									// Size Of This Pixel Format Descriptor
		  1,																// Version Number
		  PFD_DRAW_TO_WINDOW |											// Format Must Support Window
		  PFD_SUPPORT_OPENGL |											// Format Must Support OpenGL
		  PFD_DOUBLEBUFFER,												// Must Support Double Buffering
		  PFD_TYPE_RGBA,													// Request An RGBA Format
		  32,														// Select Our Color Depth
		  0, 0, 0, 0, 0, 0,												// Color Bits Ignored
		  0,														// No Alpha Buffer
		  0,																// Shift Bit Ignored
		  0,																// No Accumulation Buffer
		  0, 0, 0, 0,														// Accumulation Bits Ignored
		  24,															// 16Bit Z-Buffer (Depth Buffer)  
		  8,													// No Stencil Buffer
		  0,																// No Auxiliary Buffer
		  PFD_MAIN_PLANE,													// Main Drawing Layer
		  0,																// Reserved
		  0, 0, 0															// Layer Masks Ignored
  };
/*
  PIXELFORMATDESCRIPTOR pfd = 
  { 
    sizeof(PIXELFORMATDESCRIPTOR),  
    1,                     
    PFD_DRAW_TO_WINDOW |   
    PFD_SUPPORT_OPENGL |   
    PFD_DOUBLEBUFFER,      
    PFD_TYPE_RGBA,         
    32,                    
    0, 0, 0, 0, 0, 0,      
    8,                     
    0,                     
    0,                     
    0, 0, 0, 0,            
    24,                    
    0,                     
    0,                     
    PFD_MAIN_PLANE,        
    0,                     
    0, 0, 0                
  }; 
  */
  int pixelFormat = ChoosePixelFormat(hDC, &pfd);
  SetPixelFormat(hDC, pixelFormat, &pfd);
  hRC = wglCreateContext(hDC);
  if(!hRC) 
    return false;
  
  wglMakeCurrent(hDC, hRC);

  // open all extensions
  InitOpenGLExtensions();

  // check extensions


  if((!EXTCAPS(GL_ARB_vertex_shader))||
     (!EXTCAPS(GL_ARB_fragment_shader))||
     (!EXTCAPS(GL_ARB_shader_objects))||
     (!EXTCAPS(GL_ARB_shading_language_100))||
	 (!EXTCAPS(GL_EXT_framebuffer_object))
	 )
  {
	  MessageBox(hWnd, "Missing extensions...", "Buy new card or search for better driver :)", MB_OK);
	  return false;
  }


  shader = new GLSLShaderObject;
  shader->LoadFromFile("depth2col.frag", GL_FRAGMENT_SHADER_ARB);

  program = new CGLSLProgramObject("depth2col");
  program->AttachShader(shader);
  program->Link();
   

  glEnable(GL_DEPTH_TEST);
  glShadeModel(GL_SMOOTH);
  glMatrixMode(GL_PROJECTION);  glLoadIdentity();  
  glMatrixMode(GL_MODELVIEW);   glLoadIdentity();
  OnResize(WIDTH, HEIGHT);
  gluSphereObj = gluNewQuadric();

  GLint val;
  glGetIntegerv(GL_FRAMEBUFFER_BINDING_EXT, &val); Log.AddLine(__FILE__, "GL_FRAMEBUFFER_BINDING_EXT = %d", val);
  glGetIntegerv(GL_RENDERBUFFER_BINDING_EXT, &val); Log.AddLine(__FILE__, "GL_RENDERBUFFER_BINDING_EXT = %d", val);
  glGetIntegerv(GL_MAX_COLOR_ATTACHMENTS_EXT, &val); Log.AddLine(__FILE__, "GL_MAX_COLOR_ATTACHMENTS_EXT = %d", val);
  glGetIntegerv(GL_MAX_RENDERBUFFER_SIZE_EXT, &val); Log.AddLine(__FILE__, "GL_MAX_RENDERBUFFER_SIZE_EXT = %d", val);

  GLCALL(glGenFramebuffersEXT(1, &fb));
  GLCALL(glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fb));

  
  GLCALL(glGenTextures(1, &color_tex));
  GLCALL(glBindTexture(GL_TEXTURE_2D, color_tex));
  //GLCALL(glTexParameteri( GL_TEXTURE_2D, GL_GENERATE_MIPMAP_SGIS, GL_TRUE ));
  GLCALL(glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, TEXX, TEXY, 0, GL_RGBA, GL_UNSIGNED_BYTE, texture_data));
  GLCALL(glGenerateMipmapEXT(GL_TEXTURE_2D));
  GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR));
  GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR));
  GLCALL(glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, color_tex, 0));
  //GLCALL(glBindTexture(GL_TEXTURE_2D, 0));
  

  /*
  GLCALL(glGenTextures(1, &depth_tex));
  GLCALL(glBindTexture(GL_TEXTURE_2D, depth_tex));
  GLCALL(glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT24_ARB, TEXX, TEXY, 0, GL_DEPTH_COMPONENT, GL_FLOAT, NULL));
  GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR));
  GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR));
  GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE));
  GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE));
  GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE_ARB, GL_LUMINANCE));
  GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE_ARB));
  GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC_ARB, GL_LEQUAL));
  GLCALL(glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE));
  GLCALL(glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_TEXTURE_2D, depth_tex, 0));
  */


  //GLCALL(glGenerateMipmapEXT(GL_TEXTURE_2D));
 
  
  
  // initialize depth renderbuffer
  GLCALL(glGenRenderbuffersEXT(1, &depth_rb));
  GLCALL(glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, depth_rb));
  GLCALL(glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, TEXX, TEXY));
  GLCALL(glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, depth_rb));
  //GLCALL(glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, 0));
  



  /*
  // initialize stencil renderbuffer
  GLCALL(glGenRenderbuffersEXT(1, &stencil_rb));
  GLCALL(glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, stencil_rb));
  GLCALL(glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_STENCIL_INDEX8_EXT, TEXX, TEXY));
  //GLCALL(glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, 0));
  GLCALL(glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_STENCIL_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, stencil_rb));
 */
  
  {
	  GLenum status;                                               
	  status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);    
	  switch(status) 
	  {
		  case GL_FRAMEBUFFER_COMPLETE_EXT:
			break;
		  case GL_FRAMEBUFFER_UNSUPPORTED_EXT:
			  Log.AddLine(__FILE__, "GL_FRAMEBUFFER_UNSUPPORTED_EXT"); 
			 break;
		  default: 
			Log.AddLine(__FILE__, "ERROR %04x", status);
			return false;
	  }
  }

  GLCALL(glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0));
  

  return true;
}

std::vector<tVec3> points;
#define MAXP 2000
static double h = 0.01;
static double a = 10.0;
static double b = 28.0;
static double c = 8.0 / 3.0;
tVec3 v0={0.1, 0, 0};
tVec3 v1={0,0,0};
tVec3 eye;
float eye_rot_angle=0.0;


void RenderLorenz()
{
	glPushAttrib(GL_ALL_ATTRIB_BITS	);
	glViewport(0,0,TEXX, TEXY);
	glClearColor(0.4,0.2,0.2,0.5);
	GLCALL(glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));

	v1.x = v0.x + h * a * (v0.y - v0.x);
	v1.y = v0.y + h * (v0.x * (b - v0.z) - v0.y);
	v1.z = v0.z + h * (v0.x * v0.y - c * v0.z);

	v0 = v1;

	points.push_back(v0);
	if (points.size() > MAXP)
	{
		points.erase(points.begin());
	}

	tVec3 center;
	tVec3 pmin={1000,1000,1000}, pmax={-1000,-1000,-1000}; 

	int i;
	for (i=0; i<points.size(); i++)
	{
		if (points[i].x < pmin.x) pmin.x = points[i].x;
		if (points[i].y < pmin.y) pmin.y = points[i].y;
		if (points[i].z < pmin.z) pmin.z = points[i].z;

		if (points[i].x > pmax.x) pmax.x = points[i].x;
		if (points[i].y > pmax.y) pmax.y = points[i].y;
		if (points[i].z > pmax.z) pmax.z = points[i].z;
	}

	center.x = (pmin.x + pmax.x)/2.0f;
	center.y = (pmin.y + pmax.y)/2.0f;
	center.z = (pmin.z + pmax.z)/2.0f;

	glPushMatrix();

	GLCALL(glMatrixMode(GL_MODELVIEW));
	GLCALL(glLoadIdentity());
	eye.x = 45.0*cos(eye_rot_angle) + center.x;
	eye.z = 45.0*sin(eye_rot_angle) + center.z;
	eye.y = 15.0*sin(2.0*eye_rot_angle) + center.y;
	eye_rot_angle+= 0.02;

	gluLookAt(eye.x, eye.y, eye.z,
			 center.x, center.y, center.z,
			 0,1,0);

	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);


	
	for (i=0; i<points.size(); i++)
	{
		glPushMatrix();
		{
			glTranslatef(points[i].x, points[i].y, points[i].z);
			glColor3f(
				(sin(points[i].x/10.0)+1.0)/2.0,
				(cos(points[i].y/10.0)+1.0)/2.0,
				(sin(points[i].z/10.0)+1.0)/2.0);

			gluSphere( gluSphereObj, 0.3/*radius*/, 6/*slices*/, 6/*stacks*/);
		}
		glPopMatrix();
	}
	glPopMatrix();

	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
	glDisable(GL_COLOR_MATERIAL);
	glColor3f(1,1,1);

	glPopAttrib();
	glViewport(0,0,WIDTH, HEIGHT);
}


//--------------------------------------------------------------------------
void Render()
{   
	glClearColor(0,0,0,0);
	GLCALL(glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));

	GLCALL(glBindTexture(GL_TEXTURE_2D, 0));
	GLCALL(glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fb));
	RenderLorenz();

	GLCALL(glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0));

	GLCALL(glMatrixMode(GL_MODELVIEW));
	GLCALL(glLoadIdentity());
	GLCALL(glTranslatef( 0, 0, -orbitRadius ));
	GLCALL(glRotatef( -pitch, 1, 0, 0 ));
	GLCALL(glRotatef( -yaw,   0, 1, 0 ));
	GLCALL(glTranslatef( -pivot.x, -pivot.y, -pivot.z ));

	GLCALL(glGetFloatv(GL_MODELVIEW_MATRIX, mv));

	
	GLCALL(glBindTexture(GL_TEXTURE_2D, color_tex));
	GLCALL(glGenerateMipmapEXT(GL_TEXTURE_2D));
	glEnable(GL_TEXTURE_2D);

	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glColor3f(1,1,1);
	glBegin(GL_QUADS);
	{
		glTexCoord2f(0,0);	glVertex3f(-5, -5, 0);
		glTexCoord2f(1,0);	glVertex3f( 5, -5, 0);
		glTexCoord2f(1,1);	glVertex3f( 5,  5, 0);
		glTexCoord2f(0,1);	glVertex3f(-5,  5, 0);
	}
	glEnd();
	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);

	/* // following code doesn't work... :(
	GLCALL(glBindTexture(GL_TEXTURE_2D, depth_tex));
	glEnable(GL_TEXTURE_2D);

	program->Bind();
	program->sendUniform("depthTex", 0);
	glColor3f(1,1,1);
	glBegin(GL_QUADS);
	{
		glTexCoord2f(0,0);	glVertex3f(-5, -5, 0);
		glTexCoord2f(1,0);	glVertex3f( 5, -5, 0);
		glTexCoord2f(1,1);	glVertex3f( 5,  5, 0);
		glTexCoord2f(0,1);	glVertex3f(-5,  5, 0);
	}
	glEnd();
	program->Unbind();

	glBindTexture(GL_TEXTURE_2D, 0);
	glDisable(GL_TEXTURE_2D);
	*/
	glBegin(GL_LINE_STRIP);
	{
		glVertex3f(-5, -5, 0);
		glVertex3f( 5, -5, 0);
		glVertex3f( 5,  5, 0);
		glVertex3f(-5,  5, 0);
		glVertex3f(-5, -5, 0);
	}
	glEnd();

	SwapBuffers(hDC);  
}
